#include "petscksp.h"
#include "block.h"
#include "grid_jobs.h"
#include "invert.h"
#include "general.h"
#include <iostream>
#include <complex>
using namespace std;

int main(int argc,char **args)
{
	//PETSc Initializations
	PetscInitialize(&argc, &args, 0, 0);
	PetscInt size, rank;
	PetscErrorCode ierr;
	Mat hamiltonian, inverseHamiltonian, identityMat;
	PC pc;
	KSP ksp;
	const PetscInt rows = 3600, columns = 3600;
	
	static block hmod[300][3];
	//Set the atoms where the leads are attached.
	int isl[4] = {1, 2, 3589, 3590},
		isr[4] = {1801, 1802, 1789, 1790},
		isl180[4] = {1, 2, 3589, 3590},
		isr180[4] = {1801, 1802, 1789, 1790};
	complex<double> gl[4][4], gr[4][4], dos2;
	double a = 2, energy = -0.2, bfield = (8./32)*(0.0255664/2), coord[3600][3];
	
//******************************************************************************************************
	//Checking number of processors
	ierr = MPI_Comm_size(PETSC_COMM_WORLD, &size);CHKERRQ(ierr);
	ierr = MPI_Comm_rank(PETSC_COMM_WORLD, &rank);CHKERRQ(ierr);
	ierr = PetscSynchronizedPrintf(PETSC_COMM_WORLD, "size: %d  rank: %d\n", size, rank);CHKERRQ(ierr);
	ierr = PetscSynchronizedFlush(PETSC_COMM_WORLD);CHKERRQ(ierr);
	
	//Setup hamiltonian
	ierr = MatCreate(PETSC_COMM_WORLD, &hamiltonian);CHKERRQ(ierr);
	ierr = MatSetType(hamiltonian, MATAIJ);CHKERRQ(ierr);
	ierr = MatSetSizes(hamiltonian, PETSC_DECIDE, PETSC_DECIDE, rows, columns);CHKERRQ(ierr);
	ierr = MatAssemblyBegin(hamiltonian, MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
	ierr = MatAssemblyEnd(hamiltonian, MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
	
	//Setup PETSc Identity
	ierr = MatDuplicate(hamiltonian, MAT_DO_NOT_COPY_VALUES, &identityMat);CHKERRQ(ierr);
	ierr = MatSetType(identityMat, MATDENSE);CHKERRQ(ierr);
	for(int i=0; i<rows; i++)
	{
		ierr = MatSetValue(identityMat, i, i, 1.0, INSERT_VALUES);CHKERRQ(ierr);
	}
	ierr = MatAssemblyBegin(identityMat, MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
	ierr = MatAssemblyEnd(identityMat, MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
	
	//Setup PETSc inverseMat
	ierr = MatDuplicate(identityMat, MAT_DO_NOT_COPY_VALUES, &inverseHamiltonian);CHKERRQ(ierr);
	ierr = MatAssemblyBegin(inverseHamiltonian, MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
	ierr = MatAssemblyEnd(inverseHamiltonian, MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
	
	//Setup PETSc Solver
	ierr = KSPCreate(PETSC_COMM_WORLD, &ksp);CHKERRQ(ierr);
	ierr = KSPSetOperators(ksp, hamiltonian, hamiltonian, SAME_NONZERO_PATTERN);CHKERRQ(ierr); //Points KSP to the matrix to use
	ierr = KSPGetPC(ksp, &pc);CHKERRQ(ierr);	 //Points KSP to the PC
	ierr = PCSetType(pc, PCLU);CHKERRQ(ierr);	 //Sets the PC to PCLU
	ierr = KSPSetFromOptions(ksp);CHKERRQ(ierr); //Sets KSP with command line options
//******************************************************************************************************

	for(int i=0; i<4; i++)
	{
		for(int j=0; j<4; j++)
		{
			gl[i][j] = complex<double>(0,0);
		} //endfor
	} //endfor

	ierr = PetscPrintf(PETSC_COMM_WORLD, "bfield = %f\n", bfield);CHKERRQ(ierr);
	//cout << "bfield = " << bfield << endl;
	
	//Read in coordinates
 	set_coord(coord);
	
	init_grid0(hmod, energy, bfield, coord);
	
	for(; energy<=-0.19995; energy+=0.00005)
	{
		//Loop
		init_grid(hmod, energy, bfield, coord);	
		wiregf(a, energy, 100, 100, isl180, isr180, isl, isr, gl, gr ,coord);
		attach_leads(hmod, isl180, isr180, isl, isr, gl, gr);
		ierr = test_invert(hamiltonian, inverseHamiltonian, identityMat, pc, ksp, hmod, energy, isl, isr, bfield, gl, gr);CHKERRQ(ierr);
	}
	
	//Destroy PETSc Objects
    ierr = MatDestroy(identityMat);CHKERRQ(ierr);
    ierr = MatDestroy(inverseHamiltonian);CHKERRQ(ierr);
	//ierr = MatDestroy(hamiltonian);CHKERRQ(ierr);
	ierr = KSPDestroy(ksp);CHKERRQ(ierr);
	ierr = PetscFinalize();CHKERRQ(ierr);
	return 0;
} //end main
