//Contains general functions
//set_coord
//attach_leads
//wiregf
#ifndef GENERAL_H_
#define GENERAL_H_
#include <iostream>
#include <fstream>
#include <complex>
using namespace std;

void set_coord(double coord[3600][3])
{
	ifstream inFile("positions");
	int atomNum;
	
	if(inFile.is_open())
	{
		for(int lineNum=1; (lineNum<=3600 && !inFile.eof()); lineNum++)
		{
			inFile >> coord[lineNum-1][0] >> coord[lineNum-1][1] >> coord[lineNum-1][2] >> atomNum;
		} //endfor
		inFile.close();
	} //endif
	else cout << "Error: File not opened!\n";
} //end set_coord


void attach_leads(block stm[300][3], int isl180[], int isr180[], int isl[], int isr[],
					complex<double> gl[4][4], complex<double> gr[4][4])
{
	int temp1, temp2, temp3, temp4,
		a1bloc, a2bloc, a3bloc, a4bloc,
		a1pl, a2pl, a3pl, a4pl;
	complex<double> tempcd1, tempcd2;
	double thop = -0.25;

	for(int i=0; i<4; i++)
	{
		for(int j=0; j<4; j++)
		{
			temp1 = isl180[i]-1;
			temp2 = isl180[j]-1;
			temp3 = isr180[i]-1;
			temp4 = isr180[j]-1;

			if(temp1 >= temp2)
			{
				a1bloc = floor(temp1 / 12);
				a2bloc = get_atom_connect_block(temp2, temp1);
			} //endif
			else
			{
				a1bloc = floor(temp2 / 12);
				a2bloc = get_atom_connect_block(temp2, temp1);
			} //endelse

			if(temp3 >= temp4)
			{
				a3bloc = floor(temp3 / 12);
				a4bloc = get_atom_connect_block(temp4, temp3);
			} //endif
			else
			{
				a3bloc = floor(temp4 / 12);
				a4bloc = get_atom_connect_block(temp4, temp3);
			} //endelse

			if(a2bloc == 3)
			{
				//If in U
				// Set a2bloc and a1bloc to U's position in hmod
				a2bloc = 0;
				a1bloc = 0;
			} //endif

			if(a2bloc == 4)
			{
				a2bloc = 2;
				a1bloc = 299;
			} //endif

			if(a4bloc == 3)
			{
				a4bloc = 0;
				a3bloc = 0;
			} //endif

			if(a4bloc == 4)
			{
				a4bloc = 2;
				a3bloc = 299;
			} //endif

			a1pl  = temp1 % 12;
			a2pl  = temp2 % 12;
			a3pl  = temp3 % 12;
			a4pl  = temp4 % 12;
			tempcd1 = stm[a1bloc][a2bloc].getData(a1pl, a2pl);
			tempcd2 = stm[a3bloc][a4bloc].getData(a3pl, a4pl);

			stm[a1bloc][a2bloc].set_data(a1pl, a2pl, tempcd1 - thop*thop*gl[i][j]);
			stm[a3bloc][a4bloc].set_data(a3pl, a4pl, tempcd2 - thop*thop*gr[i][j]);
		} //endfor
	} //endfor
} //end attach_leads


void wiregf(float a, double e, float mtop, float ntop, int isl180[], int isr180[], int isl[],
			int isr[], complex<double> gl[4][4], complex<double> gr[4][4], double coord[3600][3])
{
	int ig=4;
	complex<double> zfunc, result, unitz = complex<double>(0,1);
	double yl180[ig], yr180[ig], zl180[ig], zr180[ig];
	double alpha, alpha2, fmnl, fmnr, a1x, a2x;
	double emass = .511e06, hbar = 1973.3, emn = 0, efermi = 6.0, yo = 100, zo = 4;
	double t = hbar*hbar/emass/a/a, pi = acos(-1.0);

	for(int i=0; i<ig; i++)
	{
		for(int j=0; j<ig; j++)
		{
			gl[i][j] = 0;
	      	gr[i][j] = 0;
		} //endfor
	} //endfor

	for(int i=0; i<4; i++)
	{
	    a1x = coord[isl180[i]-1][0];
	    yl180[i] = coord[isl180[i]-1][1];
	    zl180[i] = coord[isl180[i]-1][2];

	    a2x = coord[isr180[i]-1][0];
	    yr180[i] = coord[isr180[i]-1][1];
	    zr180[i] = coord[isr180[i]-1][2];
	} //endfor

	for(int i=0; i<ig; i++)
	{
		for(int j=0; j<ig; j++)
		{
			for(int m=1; m<mtop+1; m++)
			{
				for(int n=1; n<ntop+1; n++)
				{
					emn = hbar*hbar*pi*pi* (m*m/yo/yo + n*n/zo/zo) /2/emass;
					alpha = (e + efermi - emn)/t - 1;
					alpha2 = alpha * alpha;
					fmnl = sin(m*pi*yl180[i]/yo) * sin(m*pi*yl180[j]/yo) * sin(n*pi*zl180[i]/zo) * sin(n*pi*zl180[j]/zo);
					fmnr = sin(m*pi*yr180[i]/yo) * sin(m*pi*yr180[j]/yo) * sin(n*pi*zr180[i]/zo) * sin(n*pi*zr180[j]/zo);

					if(alpha < -1)
					{
						zfunc = fabs(alpha) - sqrt(alpha2 - 1);
					} //endif

					if(alpha > -1 && alpha < 1)
					{
						zfunc = -alpha + unitz*sqrt(1 - alpha2);
					} //endif

					if(alpha > 1)
					{
						zfunc = -alpha + sqrt(alpha2 - 1);
						cout << "3rd zfunc-" << zfunc << endl;
					} //endif

					gl[i][j] += fmnl*zfunc;
					gr[i][j] += fmnr*zfunc;
				} //endfor
			} //endfor
		} //endfor
	} //endfor

	for(int i=0; i<ig; i++)
	{
		for(int j=0; j<ig; j++)
		{
			  gl[i][j] *= -(8/yo/zo/a/t);
 			  gr[i][j] *= -(8/yo/zo/a/t);
		} //endfor
	} //endfor
} //end wiregf

#endif /* GENERAL_H_ */
