//Tests whether or not the matrix was set up correctly by copying it to a 3600 by 3600 matrix and inverting that.
//Results can then be compared to the old blas code.
#ifndef INVERT_H_
#define INVERT_H_
#include <iostream>
#include <complex>
#include "petscksp.h"
using namespace std;

#undef __FUNCT__
#define __FUNCT__ "test_invert"
PetscErrorCode test_invert (Mat hamiltonian, Mat inverseHamiltonian, Mat identityMat, PC pc, KSP ksp, block hmod[300][3], complex<double> ener, int isl[], int isr[],
					double bf, complex<double> gl[4][4], complex<double> gr[4][4])
{
	//PETSc Setup
	PetscErrorCode ierr;
	const PetscInt rows = 3600, columns = 3600;
	const PetscScalar *rowValues;
	PetscInt nonZeros;
	
	//cout <<"Entered Inversion!" << endl;
	int n = 3600, lda = 3600, info, ipiv[n];
	static complex<double> exmat[3600][3600];
	static complex<double> exmatc[3600][3600];
	complex<double> b[n];
	complex<double> gaml[4][4], gamrr[4][4];
	complex<double> gtl[4][4], gtr[4][4];
	complex<double> prodl[4][4], prodr[4][4], zres[4][4];
	complex<double> fermi1, fermi2, fermif, tofe;
	complex<double> doscomplex = 0, doscomplexover = 0, doscomplexdown = 0;
	double dos = 0, therm = .03, vsdmax = .1, thop = -0.25;
	double imagpart, emu1 = -vsdmax/2, emu2 = vsdmax/2;

	for(int j=0; j<300; j++)
	{
		for(int k=0; k<12; k++)
		{
			for(int l=0; l<12; l++)
			{
				exmat[k+12*j][l+12*j] = hmod[j][1].getData(k, l);
				if(hmod[j][1].getData(k, l) != complex<double>(0,0))
					ierr = MatSetValue(hamiltonian, k+12*j, l+12*j, hmod[j][1].getData(k, l), INSERT_VALUES);CHKERRQ(ierr);
			} //endfor
		} //endfor
	} //endfor

	for(int j=0; j<299; j++)
	{
		for(int k=0; k<12; k++)
		{
			for(int l=0; l<12; l++)
			{
				exmat[k+12*j][l+12*(j-1)] = hmod[j][2].getData(k, l);
				if(hmod[j][2].getData(k, l) != complex<double>(0,0))
					ierr = MatSetValue(hamiltonian, k+12*j, l+12*(j-1), hmod[j][2].getData(k, l), INSERT_VALUES);CHKERRQ(ierr);
				if(j == 298)
				{
					exmat[n-12][n-23] = hmod[j][2].getData(0, 1);
					if(hmod[j][2].getData(0, 1) != complex<double>(0,0))
						ierr = MatSetValue(hamiltonian, n-12, n-23, hmod[j][2].getData(0, 1), INSERT_VALUES);CHKERRQ(ierr);
					
					exmat[n-9][n-22] = hmod[j][2].getData(3, 2);
					if(hmod[j][2].getData(3, 2) != complex<double>(0,0))
						ierr = MatSetValue(hamiltonian, n-9, n-22, hmod[j][2].getData(3, 2), INSERT_VALUES);CHKERRQ(ierr);
					
					exmat[n-8][n-19] = hmod[j][2].getData(4, 5);
					if(hmod[j][2].getData(4, 5) != complex<double>(0,0))
						ierr = MatSetValue(hamiltonian, n-8, n-19, hmod[j][2].getData(4, 5), INSERT_VALUES);CHKERRQ(ierr);
					
					exmat[n-5][n-18] = hmod[j][2].getData(7, 6);
					if(hmod[j][2].getData(7, 6) != complex<double>(0,0))
						ierr = MatSetValue(hamiltonian, n-5, n-18, hmod[j][2].getData(7, 6), INSERT_VALUES);CHKERRQ(ierr);
					
					exmat[n-4][n-15] = hmod[j][2].getData(8, 9);
					if(hmod[j][2].getData(8, 9) != complex<double>(0,0))
						ierr = MatSetValue(hamiltonian, n-4, n-15, hmod[j][2].getData(8, 9), INSERT_VALUES);CHKERRQ(ierr);
					
					exmat[n-1][n-14] = hmod[j][2].getData(11, 10);
					if(hmod[j][2].getData(11, 10) != complex<double>(0,0))
						ierr = MatSetValue(hamiltonian, n-1, n-14, hmod[j][2].getData(11, 10), INSERT_VALUES);CHKERRQ(ierr);
				} //endif
			} //endfor
		} //endfor
	} //endfor

	for(int j=1; j<300; j++)
	{
		for(int k=0; k<12; k++)
		{
			for(int l=0; l<12; l++)
			{
				exmat[k+12*(j-1)][l+12*j] = hmod[j][0].getData(k, l);
				if(hmod[j][0].getData(k, l) != complex<double>(0,0))
					ierr = MatSetValue(hamiltonian, k+12*(j-1), l+12*j, hmod[j][0].getData(k, l), INSERT_VALUES);CHKERRQ(ierr);
			} //endfor
		} //endfor
	} //endfor

	for(int k=0; k<12; k++)
	{
		for (int l=0; l<12; l++)
		{
			exmat[k][l+12*299] = hmod[0][0].getData(k, l);
			if(hmod[0][0].getData(k, l) != complex<double>(0,0))
				ierr = MatSetValue(hamiltonian, k, l+12*299, hmod[0][0].getData(k, l), INSERT_VALUES);CHKERRQ(ierr);
			
			exmat[k+12*299][l] = hmod[299][2].getData(k, l);
			if(hmod[299][2].getData(k, l) != complex<double>(0,0))
				ierr = MatSetValue(hamiltonian, k+12*299, l, hmod[299][2].getData(k, l), INSERT_VALUES);CHKERRQ(ierr);
		} //endfor
	} //endfor

	for(int i=0; i<n-11; i++)
	{
		doscomplex += exmatc[i][i];
		doscomplexover += exmatc[i][i+11];
		doscomplexdown += exmatc[i+11][i];
	} //endfor

	dos = 0;
//******************************************************************************************************
	//Setup PETSc Hamiltonian
	ierr = MatAssemblyBegin(hamiltonian, MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
	ierr = MatAssemblyEnd(hamiltonian, MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
	
	//Setup PETSc Solver
	ierr = KSPSetOperators(ksp, hamiltonian, hamiltonian, DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
	ierr = KSPSetUp(ksp);CHKERRQ(ierr);
	
	//Factor hamiltonian and solve hamiltonian * inverseHamiltonian = identityMat for inverseHamiltonian
	ierr = PCFactorGetMatrix(pc, &hamiltonian);CHKERRQ(ierr);
	ierr = MatMatSolve(hamiltonian, identityMat, inverseHamiltonian);CHKERRQ(ierr);
	
	//Retrieve inverse
	for(int i=0; i<rows; i++)
	{
		ierr = MatGetRow(inverseHamiltonian, i, &nonZeros, PETSC_NULL, &rowValues);CHKERRQ(ierr);
		for(int j=0; j<columns; j++)
		{
			exmat[i][j] = rowValues[j];
		}
		ierr = MatRestoreRow(inverseHamiltonian, i, &nonZeros, PETSC_NULL, &rowValues);CHKERRQ(ierr);
	}
	
	//Reset hamiltonian
	//ierr = MatSetUnfactored(hamiltonian);CHKERRQ(ierr);
	//ierr = MatZeroEntries(hamiltonian);CHKERRQ(ierr);
//******************************************************************************************************

	for(int i=0; i<n; i++)
	{
		imagpart = exmat[i][i].imag();
		dos += -1.0 * imagpart / 3.141592635;
	} //endfor

	for(int i=0; i<4; i++)
	{
		for(int j=0; j<4; j++)
		{
			gaml[i][j]  = exmat[isl[i]-1][isr[j]-1];
			gamrr[i][j] = conj(exmat[isr[j]-1][isl[i]-1]);
			gtr[i][j] = -gr[i][j].imag();
			gtl[i][j] = -gl[i][j].imag();
		} //endfor
	} //endfor

	//NOTE: We need to multiply the previous four 4x4 matrices together to get tofe
	for(int i=0; i<4; i++)
	{
		for (int j=0; j<4; j++)
		{
			prodl[i][j] = complex<double>(0,0);
			prodr[i][j] = complex<double>(0,0);
			for(int k=0; k<4; k++)
			{
				prodl[i][j] += gtl[i][k] * gaml[k][j];
				prodr[i][j] += gtr[i][k] * gamrr[k][j];
			} //endfor
		} //endfor
	} //endfor

	for(int i=0; i<4; i++)
	{
		for(int j=0; j<4; j++)
		{
			zres[i][j] = complex<double>(0,0);
			for(int k=0; k<4; k++)
			{
				zres[i][j] += prodl[i][k] * prodr[k][j];
			} //endfor
		} //endfor
	} //endfor

	for(int i=0; i<4; i++)
	{
		tofe += zres[i][i];
	} //endfor

	fermi1 = 1.0 / (exp((ener-emu1)/therm)+1.0);
	fermi2 = 1.0 / (exp((ener-emu2)/therm)+1.0);
	fermif = 2 * abs(tofe) * (fermi1-fermi2);

	cout << ener << "  " << dos << " " << 4*thop*thop*thop*thop*tofe << fermif << endl;
	
	PetscFunctionReturn(0);
} //end test_invert

#endif /* INVERT_H_ */
